Method: Origen::RevisionControl::Base#diff_cmd

Defined in:
lib/origen/revision_control/base.rb

#diff_cmd(file, version) ⇒ Object

Returns the command the user must run to execute a diff of the current version of the given file against the given version of it.

Parameters:

  • file (String, Pathname)

    The local path to the file to be compared.

  • version (String)

    The version of the file to compare to.



166
167
168
# File 'lib/origen/revision_control/base.rb', line 166

def diff_cmd(file, version)
  fail "The #{self.class} driver does not support the diff_cmd method!"
end