Class: Origen::Application

Inherits:
Object show all
Includes:
Users
Defined in:
lib/origen/application.rb,
lib/origen/application/lsf.rb,
lib/origen/application/runner.rb,
lib/origen/application/target.rb,
lib/origen/application/plugins.rb,
lib/origen/application/release.rb,
lib/origen/application/deployer.rb,
lib/origen/application/statistics.rb,
lib/origen/application/environment.rb,
lib/origen/application/lsf_manager.rb,
lib/origen/application/configuration.rb,
lib/origen/application/plugins_manager.rb,
lib/origen/application/version_tracker.rb,
lib/origen/application/workspace_manager.rb,
lib/origen/application/command_dispatcher.rb

Overview

In Origen v2 this class was introduced to formally co-ordinate application level configuration of Origen.

Configuration

See Origen::Application::Configuration for the available options.

Direct Known Subclasses

OrigenGlobalApplication

Defined Under Namespace

Classes: CommandDispatcher, Configuration, Deployer, Environment, LSF, LSFManager, Plugins, PluginsManager, RakeLoader, Release, Runner, Statistics, Target, VersionTracker, WorkspaceManager

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Users

#admins, #app_users, #current_user

Instance Attribute Details

#nameObject

Returns the name of the given application, this is the name that will be used to refer to the application when it is used as a plugin



645
646
647
648
649
650
651
652
# File 'lib/origen/application.rb', line 645

def name
  raw_name = (@name ||= namespace)
  unless defined?(@normalized_name_source) && @normalized_name_source == raw_name
    @normalized_name_source = raw_name
    @normalized_name = raw_name.to_s.underscore.symbolize
  end
  @normalized_name
end

#namespaceObject

Returns the namespace used by the application as a string



306
307
308
# File 'lib/origen/application.rb', line 306

def namespace
  @namespace ||= self.class.to_s.split('::').first.gsub('_', '').sub('Application', '')
end

Class Method Details

.from_namespace(item) ⇒ Object

Returns the application instance (i.e. main app or the plugin) that owns the given class/module (literal, string or symbol representation is accepted) or object instance. Returns nil if no matching Origen application can be found.

Origen::Application.from_namespace(MyApp)                       # => <my_app instance>
Origen::Application.from_namespace(MyApp::MyClass)              # => <my_app instance>
Origen::Application.from_namespace('MyApp::MyClass')            # => <my_app instance>
Origen::Application.from_namespace(<my_app::my_class instance>) # => <my_app instance>


75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
# File 'lib/origen/application.rb', line 75

def from_namespace(item)
  unless item.is_a?(String)
    if item.is_a?(Module) || item.is_a?(Class) || item.is_a?(Symbol)
      item = item.to_s
    else # Assume to be an object instance in this case
      item = item.class.to_s
    end
  end
  namespace = item.split('::').first
  @apps_by_namespace ||= {}
  @apps_by_namespace[namespace] ||= begin
    return Origen.app if Origen.app.namespace == namespace

    Origen.app.plugins.each do |plugin|
      return plugin if plugin.namespace == namespace
    end
    nil
  end
end

.inherited(base) ⇒ Object



30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
# File 'lib/origen/application.rb', line 30

def inherited(base)
  # Somehow using the old import system and version file format we can get in here when
  # loading the version, this can be removed in future when the imports API is retired
  unless caller[0] =~ /version.rb.*/
    if base.to_s == 'OrigenGlobalApplication'
      app = base.instance
      app.root = Origen.root
      Origen.register_application(app)
    else
      root = Pathname.new(caller[0].sub(/(\\|\/)?config(\\|\/)application.rb.*/, '')).realpath
      app = base.instance
      app.root = root.to_s
      if Origen.plugins_loaded? && !Origen.loading_top_level?
        # This situation of a plugin being loaded after the top-level app could occur if the app
        # doesn't require the plugin until later, in that case there is nothing the plugin owner
        # can do and we just need to accept that this can happen.
        # Origen.log.warning "The #{app.name} plugin is using a non-standard loading mechanism, upgrade to a newer version of it to get rid of this warning (please report a bug to its owner if this warning persists)"
        Origen.register_application(app)
        # Origen.app.plugins << app
      else
        Origen.register_application(app)
      end
      app.add_lib_to_load_path!
      # Also blow this cache as a new app has just been added
      @apps_by_namespace = nil
    end
  end
end

.instanceObject



59
60
61
# File 'lib/origen/application.rb', line 59

def instance
  @instance ||= new
end

.respond_to?(*args) ⇒ Boolean

Returns:

  • (Boolean)


63
64
65
# File 'lib/origen/application.rb', line 63

def respond_to?(*args)
  super || instance.respond_to?(*args)
end

Instance Method Details

#_add_block_files(files, block_dir, current_dir, sub_block = false) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.



165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
# File 'lib/origen/application.rb', line 165

def _add_block_files(files, block_dir, current_dir, sub_block = false)
  fields = current_dir.relative_path_from(block_dir).to_s.split('/')
  fields.delete('derivatives')
  fields.delete('sub_blocks')
  path = fields.join('/')
  files[path] ||= {}
  files[path][:_sub_block] = true if sub_block
  Dir.glob(current_dir.join('*.rb')).each do |file|
    file = Pathname.new(file)
    type = file.basename('.rb').to_s.to_sym
    unless type == :model || type == :controller
      files[path][type] ||= []
      files[path][type] << file.to_s
    end
  end
  derivatives = current_dir.join('derivatives')
  if derivatives.exist?
    derivatives.children.each do |item|
      if item.directory?
        _add_block_files(files, block_dir, item)
      end
    end
  end
  sub_blocks = current_dir.join('sub_blocks')
  if sub_blocks.exist?
    sub_blocks.children.each do |item|
      if item.directory?
        _add_block_files(files, block_dir, item, true)
      end
    end
  end
end

#add_callback_listener(obj) ⇒ Object



760
761
762
# File 'lib/origen/application.rb', line 760

def add_callback_listener(obj)
  dynamic_resource(:callback_listeners, [], adding: true) << obj
end

#add_config_attribute(*args) ⇒ Object



639
640
641
# File 'lib/origen/application.rb', line 639

def add_config_attribute(*args)
  Application::Configuration.add_attribute(*args)
end

#add_lib_to_load_path!Object

This method is called just after an application inherits from Origen::Application, allowing the developer to load classes in lib and use them during application configuration.

class MyApplication < Origen::Application
require "my_backend" # in lib/my_backend
config.i18n.backend = MyBackend
end


1058
1059
1060
1061
1062
# File 'lib/origen/application.rb', line 1058

def add_lib_to_load_path! # :nodoc:
  [root.join('lib'), root.join('vendor', 'lib'), root.join('app', 'lib')].each do |path|
    $LOAD_PATH.unshift(path.to_s) if File.exist?(path) && !$LOAD_PATH.include?(path.to_s)
  end
end

#add_persistant_callback_listener(obj) ⇒ Object



764
765
766
767
768
# File 'lib/origen/application.rb', line 764

def add_persistant_callback_listener(obj)
  @persistant_callback_listeners ||= []
  @persistant_callback_listeners << obj
  @persistant_callback_listeners.uniq!
end

#add_pin_to_pin_map(id, pin) ⇒ Object



815
816
817
818
819
820
821
822
823
# File 'lib/origen/application.rb', line 815

def add_pin_to_pin_map(id, pin)
  # If being added during target load...
  if @load_event
    pin_map[id] = pin
  # Being added late in the process...
  else
    @transient_resources[:pin_map][id] = pin
  end
end

#add_pingroup_to_pingroup_map(id, pins) ⇒ Object



829
830
831
832
833
834
835
836
837
# File 'lib/origen/application.rb', line 829

def add_pingroup_to_pingroup_map(id, pins)
  # If being added during target load...
  if @load_event
    pingroup_map[id] = pins
  # Being added late in the process...
  else
    @transient_resources[:pingroup_map][id] = pins
  end
end

#add_toplevel_listener(obj) ⇒ Object



770
771
772
773
774
775
776
777
# File 'lib/origen/application.rb', line 770

def add_toplevel_listener(obj)
  if Origen.top_level
    puts "Attempt to set an instance of #{obj.class} as the top level when there is already an instance of #{Origen.top_level.class} defined as the top-level!"
    fail 'Only one object that include the Origen::TopLevel module can be instantiated per target!'
  end
  $dut = obj
  dynamic_resource(:toplevel_listeners, [], adding: true) << obj
end

#author(version = Origen.app.version.prefixed) ⇒ Object

Returns the author (committer) for the current or given application version

If the user can be found in the directory then a user object will be returned, otherwise the name will be returned as a String



562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
# File 'lib/origen/application.rb', line 562

def author(version = Origen.app.version.prefixed)
  version = VersionString.new(version)
  version = version.prefixed if version.semantic?
  capture = false
  File.readlines("#{Origen.root}/doc/history").each do |line|
    line = line.strip
    if capture
      if capture && line =~ /^#+ by (.*) on (.*(AM|PM))/
        user = Origen.fsl.find_by_name(Regexp.last_match(1))
        return user if user

        return Regexp.last_match(1)
      end
    else
      if line =~ /Tag:/
        line = line.gsub('\\', '')
        if line =~ /^#+ Tag: #{version}$/ ||
           line =~ />Tag: #{version}</
          capture = true
        end
      end
    end
  end
  nil
end

#blocks_filesObject

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns a lookup table for all block definitions (app/blocks) that the app contains



149
150
151
152
153
154
155
156
157
158
159
160
161
162
# File 'lib/origen/application.rb', line 149

def blocks_files
  @blocks_files ||= begin
    files = {}
    block_dir = Pathname.new(File.join(root, 'app', 'blocks'))
    if block_dir.exist?
      block_dir.children.each do |item|
        if item.directory?
          _add_block_files(files, block_dir, item)
        end
      end
    end
    files
  end
end

#branch(version = Origen.app.version.prefixed) ⇒ Object Also known as: selector

Returns the branch for the current or given application version



589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
# File 'lib/origen/application.rb', line 589

def branch(version = Origen.app.version.prefixed)
  version = VersionString.new(version)
  version = version.prefixed if version.semantic?
  capture = false
  File.readlines("#{Origen.root}/doc/history").each do |line|
    line = line.strip
    if capture
      if capture && line =~ /^#+ .*(Selector|Branch): '(.*)'/
        return Regexp.last_match(2).gsub('\\', '')
      end
    else
      if line =~ /Tag:/
        line = line.gsub('\\', '')
        if line =~ /^#+ Tag: #{version}$/ ||
           line =~ />Tag: #{version}</
          capture = true
        end
      end
    end
  end
  nil
end

#callback_listenersObject



745
746
747
748
749
750
# File 'lib/origen/application.rb', line 745

def callback_listeners
  current = Origen.app.plugins.current
  applications = [self]
  applications << current if current
  applications + instantiated_callback_listeners
end

#clear_dynamic_resources(type = :transient) ⇒ Object



984
985
986
987
988
989
990
991
# File 'lib/origen/application.rb', line 984

def clear_dynamic_resources(type = :transient)
  @top_level = nil
  if type == :transient
    @transient_resources = nil
  else
    @static_resources = nil
  end
end

#configObject



635
636
637
# File 'lib/origen/application.rb', line 635

def config
  @config ||= Configuration.new(self)
end

#contributorsObject



625
626
627
628
629
630
631
632
633
# File 'lib/origen/application.rb', line 625

def contributors
  c = []
  File.readlines("#{Origen.root}/doc/history").each do |line|
    if line =~ /^#+ by (.*) on /
      c << Regexp.last_match(1)
    end
  end
  c.uniq
end

#current?Boolean Also known as: standalone?, running_standalone?

Returns true if the given application instance is the current top level application

Returns:

  • (Boolean)


386
387
388
389
390
391
392
393
394
# File 'lib/origen/application.rb', line 386

def current?
  # If this is called before the plugins are loaded (i.e. by a plugin's application file), then
  # it is definitely not the top-level app
  if Origen.application_loaded?
    Origen.app == self
  else
    Origen.root == root
  end
end

#current_jobObject



198
199
200
# File 'lib/origen/application.rb', line 198

def current_job
  current_jobs.last
end

#current_jobsObject



202
203
204
# File 'lib/origen/application.rb', line 202

def current_jobs
  @current_jobs ||= []
end

#current_plugin?Boolean

Returns true if the given application instance is the current plugin

Returns:

  • (Boolean)


400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
# File 'lib/origen/application.rb', line 400

def current_plugin?
  if Origen.application_loaded?
    Origen.app.plugins.current == self
  else
    puts <<-END
current_plugin? cannot be used at this point in your code since the app is not loaded yet.

If you are calling this from config/application.rb then you can only use current_plugin? within a block:

# Not OK
if current_plugin?
  config.output_directory = "#{Origen.root}/output/dir1"
else
  config.output_directory = "#{Origen.root}/output/dir2"
end

# OK
config.output_directory do
  if current_plugin?
"#{Origen.root}/output/dir1"
  else
"#{Origen.root}/output/dir2"
  end
end

END
    exit 1
  end
end

#dbObject



709
710
711
# File 'lib/origen/application.rb', line 709

def db
  @db ||= Database::KeyValueStores.new(self)
end

#deployerObject



689
690
691
# File 'lib/origen/application.rb', line 689

def deployer
  @deployer ||= Deployer.new
end

#dynamic_resource(name, default, options = {}) ⇒ Object

Enable for debugging to see what the currently tracked objects are def object_store [@load_event, @static_resources, @transient_resources] end



957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
# File 'lib/origen/application.rb', line 957

def dynamic_resource(name, default, options = {})
  @static_resources ||= {}
  @transient_resources ||= {}
  if @load_event == :static ||
     (!@load_event && options[:adding])
    if options[:set]
      @static_resources[name] = default
    else
      @static_resources[name] ||= default
    end
  elsif @load_event == :transient
    if options[:set]
      @transient_resources[name] = default
    else
      @transient_resources[name] ||= default
    end
  else
    static = @static_resources[name] ||= default
    transient = @transient_resources[name] ||= default
    if static.respond_to?('+')
      static + transient
    else
      static.merge(transient)
    end
  end
end

#environmentObject



677
678
679
# File 'lib/origen/application.rb', line 677

def environment
  @environment ||= Environment.new
end

#fail(message: nil, exception_class: RuntimeError) ⇒ Object

Prepends the application name to the fail message and throws a RuntimeError exception. Very similar to the plain fail method with the addition of prepending the application name. Prepended message: 'Fail in app.name: ' If no message if provided, message is set to 'Fail in app.name'

Parameters:

  • message (String) (defaults to: nil)

    Message to print with the exception. If the message option is nil, a default message will be used instead.

  • exception_class (Class) (defaults to: RuntimeError)

    Custom Exception class to throw. May require the full namespace, e.g. Origen::OrigenError instead of just OrigenError.

Raises:

  • (RuntimeError, exception_class)

    Option exception_class is raised, defaulting to RuntimeError.



1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
# File 'lib/origen/application.rb', line 1014

def fail(message: nil, exception_class: RuntimeError)
  message.nil? ? message = "Fail in #{name}" : message = "Fail in #{name}: #{message}"
  e = exception_class.new(message)

  # If the caller is Origen.app.fail!, remove this caller from the backtrace, leaving where Origen.app.fail! was called.
  # As an aside, if there's an exception raised in Origen.app.fail!, then that would actually raise a Kernel.fail, so there's no concern with masking
  # out a problem with Origen.app.fail! by doing this.
  if caller[0] =~ (/lib\/origen\/application.rb:\d+:in `fail!'/)
    e.set_backtrace(caller[1..-1])
  else
    e.set_backtrace(caller)
  end
  Kernel.fail(e)
end

#fail!(message: nil, exception_class: RuntimeError, exit_status: 1) ⇒ Object

Similar to Origen.app.fail, but will instead print the message using Origen.log.error and exit the current process (using exit 1) UNLESS --debug is used. In those cases, exit will not be used and instead this will behave the same as #fail. Purpose here is to allow fail! for normal usage, but provide more details as to where fail! was used when running in debug.

Parameters:

  • message (String) (defaults to: nil)

    Message to print with the exception. If the message option is nil, a default message will be used instead.

  • exception_class (Class) (defaults to: RuntimeError)

    Custom Exception class to throw. May require the full namespace.

  • exit_status (Integer) (defaults to: 1)

    Exit status to use when exiting the application.

Raises:

  • (RuntimeError, SystemExit, exception_class)

    When debug is disabled, SystemExit will be raised. When debug is enabled, exception_class will be raised, defaulting to RuntimeError.



1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
# File 'lib/origen/application.rb', line 1037

def fail!(message: nil, exception_class: RuntimeError, exit_status: 1)
  if Origen.debug?
    self.fail(message: message, exception_class: exception_class)
  else
    begin
      self.fail(message: message, exception_class: exception_class)
    rescue exception_class => e
      Origen.log.error(e.message)
      exit exit_status
    end
  end
end

#gem_nameObject



654
655
656
# File 'lib/origen/application.rb', line 654

def gem_name
  (Origen.app.config.gem_name || name).to_s.underscore.symbolize
end

#imports_directoryObject Also known as: imports_dir

Returns a path to the imports directory (e.g. used by the remotes and similar features) for the application. e.g. if the app live at /home/thao/my_app, then the imports directory will typically be /home/thao/.my_app_imports_DO_NOT_HAND_MODIFY

Origen will ensure that this directory is outside of the scope of the current application's revision control system. This prevents conflicts with the revision control system for the application and those used to import 3rd party dependencies



288
289
290
# File 'lib/origen/application.rb', line 288

def imports_directory
  workspace_manager.imports_directory
end

#inspectObject



261
262
263
# File 'lib/origen/application.rb', line 261

def inspect
  "<Origen app (#{name}):#{object_id}>"
end

#instantiated_callback_listenersObject



752
753
754
# File 'lib/origen/application.rb', line 752

def instantiated_callback_listeners
  dynamic_resource(:callback_listeners, []) + (@persistant_callback_listeners || [])
end

#listeners_for(*args) ⇒ Object



438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
# File 'lib/origen/application.rb', line 438

def listeners_for(*args)
  callback = args.shift
  max = args.first.is_a?(Numeric) ? args.shift : nil
  options = args.shift || {}
  options = {
    top_level: :first
  }.merge(options)
  listeners = callback_listeners
  if Origen.top_level
    listeners -= [Origen.top_level.model]
    if options[:top_level]
      if options[:top_level] == :last
        listeners = listeners + [Origen.top_level]
      else
        listeners = [Origen.top_level] + listeners
      end
    end
  end
  listeners = listeners.select { |l| l.respond_to?(callback) }.map do |l|
    if l.try(:is_an_origen_model?)
      l.respond_to_directly?(callback) ? l : l.controller
    else
      l
    end
  end
  if max && listeners.size > max
    fail "You can only define a #{callback} callback #{max > 1 ? (max.to_s + 'times') : 'once'}, however you have declared it #{listeners.size} times for instances of: #{listeners.map(&:class)}"
  end

  listeners
end

#load_event(type) ⇒ Object



993
994
995
996
997
# File 'lib/origen/application.rb', line 993

def load_event(type)
  @load_event = type
  yield
  @load_event = nil
end

#load_target!(options = {}) ⇒ Object



863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
# File 'lib/origen/application.rb', line 863

def load_target!(options = {})
  options = {
    force_debug: false
  }.merge(options)
  @on_create_called = false
  @target_loading = true
  if options[:reload]
    @target_load_count = 0
  else
    @target_load_count ||= 0
    @target_load_count += 1
  end
  listeners_for(:before_load_target).each(&:before_load_target)
  # Remember these if the target has to be reloaded
  @target_load_options = options.merge({})
  # Since this is a load it will re-instantiate any objects that the application
  # declares here, the objects registered with origen should be refreshed accordingly
  clear_dynamic_resources
  load_event(:transient) do
    Origen.mode = Origen.app.session.origen_core[:mode] || :production # Important since a production target may rely on the default
    begin
      $_target_options = @target_load_options
      Origen.target.set_signature(@target_load_options)
      $dut = nil
      load environment.file if environment.file
      if target.proc
        target.proc.call
      else
        load target.file!
      end
    ensure
      $_target_options = nil
    end
    @target_instantiated = true
    Origen.mode = :debug if options[:force_debug] || ENV['ORIGEN_DEBUG']
    listeners_for(:on_create).each do |obj|
      unless obj.is_a?(Origen::SubBlocks::Placeholder)
        if obj.try(:is_a_model_and_controller?)
          m = obj.model
          c = obj.controller
          m.on_create if m.respond_to_directly?(:on_create)
          c.on_create if c.respond_to_directly?(:on_create)
        else
          obj.on_create
        end
      end
    end
    @on_create_called = true
    # Keep this within the load_event to ensure any objects that are further instantiated objects
    # will be associated with (and cleared out upon reload of) the current target
    listeners_for(:on_load_target).each(&:on_load_target)
  end
  listeners_for(:after_load_target).each(&:after_load_target)
  Origen.app.plugins.validate_production_status
  @target_loading = false
end

#load_tasksObject

Load all rake tasks defined in the application's lib/task directory



207
208
209
# File 'lib/origen/application.rb', line 207

def load_tasks
  RakeLoader.new.load_tasks
end

#lsfObject



681
682
683
# File 'lib/origen/application.rb', line 681

def lsf
  @lsf ||= LSF.new
end

#lsf_managerObject



693
694
695
# File 'lib/origen/application.rb', line 693

def lsf_manager
  @lsf_manager ||= LSFManager.new
end

#mailerObject



705
706
707
# File 'lib/origen/application.rb', line 705

def mailer
  @mailer ||= Utility::Mailer.new
end

#maillist_devObject

Returns array of email addresses in the DEV maillist file



311
312
313
# File 'lib/origen/application.rb', line 311

def maillist_dev
  maillist_parse(maillist_dev_file)
end

#maillist_dev_fileObject

Returns default location of DEV maillist file (customize locally if needed)



321
322
323
# File 'lib/origen/application.rb', line 321

def maillist_dev_file
  Origen.app.root.to_s + '/config/maillist_dev.txt'
end

#maillist_parse(file) ⇒ Object

Parses maillist file and returns an array of email address



331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
# File 'lib/origen/application.rb', line 331

def maillist_parse(file)
  maillist = []

  # if file doesn't exist, just return empty array, otherwise, parse for emails
  if File.exist?(file)
    File.readlines(file).each do |line|
      if index = (line =~ /\#/)
        # line contains some kind of comment
        # check if there is any useful info, ignore it not
        unless line[0, index].strip.empty?
          maillist << Origen::Users::User.new(line[0, index].strip).email
        end
      else
        # if line is not empty, generate an email
        unless line.strip.empty?
          maillist << Origen::Users::User.new(line.strip).email
        end
      end
    end
  end
  maillist
end

#maillist_prodObject

Returns array of email addresses in the PROD maillist file



316
317
318
# File 'lib/origen/application.rb', line 316

def maillist_prod
  maillist_parse(maillist_prod_file)
end

#maillist_prod_fileObject

Returns default location of PROD maillist file (customize locally if needed)



326
327
328
# File 'lib/origen/application.rb', line 326

def maillist_prod_file
  Origen.app.root.to_s + '/config/maillist_prod.txt'
end

#on_create_called?Boolean

Returns true if the on_create callback has already been called during a target load

Returns:

  • (Boolean)


921
922
923
# File 'lib/origen/application.rb', line 921

def on_create_called?
  !!@on_create_called
end

#on_loadedObject

This callback handler will fire once the main app and all of its plugins have been loaded



252
253
254
# File 'lib/origen/application.rb', line 252

def on_loaded
  config.log_deprecations
end

#origen_core?Boolean

Convenience method to check if the given application instance is Origen core

Returns:

  • (Boolean)


257
258
259
# File 'lib/origen/application.rb', line 257

def origen_core?
  name.to_s.symbolize == :origen_core
end

#pattern_iteratorsObject



741
742
743
# File 'lib/origen/application.rb', line 741

def pattern_iterators
  @pattern_iterators ||= []
end

#pin_mapObject



811
812
813
# File 'lib/origen/application.rb', line 811

def pin_map
  dynamic_resource(:pin_map, {})
end

#pin_namesObject



855
856
857
858
859
860
861
# File 'lib/origen/application.rb', line 855

def pin_names
  if @load_event
    dynamic_resource(:pin_names, {})
  else
    @transient_resources[:pin_names] ||= {}
  end
end

#pin_pattern_excludeObject



847
848
849
850
851
852
853
# File 'lib/origen/application.rb', line 847

def pin_pattern_exclude
  if @load_event
    dynamic_resource(:pin_pattern_exclude, [])
  else
    @transient_resources[:pin_pattern_exclude] ||= []
  end
end

#pin_pattern_orderObject



839
840
841
842
843
844
845
# File 'lib/origen/application.rb', line 839

def pin_pattern_order
  if @load_event
    dynamic_resource(:pin_pattern_order, [])
  else
    @transient_resources[:pin_pattern_order] ||= []
  end
end

#pingroup_mapObject



825
826
827
# File 'lib/origen/application.rb', line 825

def pingroup_map
  dynamic_resource(:pingroup_map, {})
end

#pluginsObject



658
659
660
661
662
663
664
# File 'lib/origen/application.rb', line 658

def plugins
  if Origen.app_loaded?
    @plugins ||= Plugins.new
  else
    Plugins.new
  end
end

#plugins_managerObject Also known as: plugin_manager, current_plugin



666
667
668
669
# File 'lib/origen/application.rb', line 666

def plugins_manager
  Origen.deprecated 'Origen.app.plugins_manager and Origen.app.current_plugin are deprecated, use Origen.app.plugins instead'
  plugins
end

#previous_versionsObject



613
614
615
616
617
618
619
620
621
622
623
# File 'lib/origen/application.rb', line 613

def previous_versions
  versions = []
  File.readlines("#{Origen.root}/doc/history").each do |line|
    line = line.strip
    if line =~ /^#+ Tag: (.*)$/ ||
       line =~ />Tag: ([^<]*)</
      versions << Regexp.last_match(1).gsub('\\', '')
    end
  end
  versions.uniq
end

#release(options) ⇒ Object



731
732
733
734
# File 'lib/origen/application.rb', line 731

def release(options)
  @release ||= Release.new
  @release.run(options)
end

#release_date(version = Origen.app.version.prefixed) ⇒ Object

Returns the release date for the current or given application version



529
530
531
532
# File 'lib/origen/application.rb', line 529

def release_date(version = Origen.app.version.prefixed)
  time = release_time(version)
  time ? time.to_date : nil
end

#release_note(version = Origen.app.version.prefixed) ⇒ Object

Returns the release note for the current or given application version



492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
# File 'lib/origen/application.rb', line 492

def release_note(version = Origen.app.version.prefixed)
  version = VersionString.new(version)
  version = version.prefixed if version.semantic?
  capture = false
  note_started = false
  note = []
  File.readlines("#{Origen.root}/doc/history").each do |line|
    line = line.strip
    if capture
      if note_started
        if line =~ /^<a class="anchor release_tag/ || line =~ /^#+ Tag/
          note.pop while note.last && note.last.empty?
          return note
        end
        if line.empty? && note.empty?
          # Don't capture preceding blank lines
        else
          note << line
        end
      elsif line =~ /^#+ by/
        note_started = true
      end
    else
      if line =~ /Tag:/
        line = line.gsub('\\', '')
        if line =~ /^#+ Tag: #{version}$/ ||
           line =~ />Tag: #{version}</
          capture = true
        end
      end
    end
  end
  note.pop while note.last && note.last.empty?
  note
end

#release_time(version = Origen.app.version.prefixed) ⇒ Object

Returns the release time for the current or given application version



535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
# File 'lib/origen/application.rb', line 535

def release_time(version = Origen.app.version.prefixed)
  version = VersionString.new(version)
  version = version.prefixed if version.semantic?
  capture = false
  File.readlines("#{Origen.root}/doc/history").each do |line|
    line = line.strip
    if capture
      if capture && line =~ /^#+ by .* on (.*(AM|PM))/
        return Time.parse(Regexp.last_match(1))
      end
    else
      if line =~ /Tag:/
        line = line.gsub('\\', '')
        if line =~ /^#+ Tag: #{version}$/ ||
           line =~ />Tag: #{version}</
          capture = true
        end
      end
    end
  end
  nil
end

#reload_target!(options = {}) ⇒ Object

Equivalent to load_target! except that any options that were passed to load_target! the last time it was called will be re-applied when (re)loading the target.



938
939
940
941
942
943
944
945
946
# File 'lib/origen/application.rb', line 938

def reload_target!(options = {})
  old_options = @target_load_options || {}
  options = (@target_load_options || {}).merge(options)
  if options[:skip_first_time] && @target_load_count == 1
    @target_load_count += 1
  else
    load_target!(options.merge(reload: true))
  end
end

#remotes_directoryObject Also known as: remotes_dir

Returns a path to the remotes directory for the application. e.g. if the app live at /home/thao/my_app, then the remotes directory will typically be /home/thao/.my_app_remotes_DO_NOT_HAND_MODIFY

Origen will ensure that this directory is outside of the scope of the current application's revision control system. This prevents conflicts with the revision control system for the application and those used to import 3rd party dependencies



300
301
302
# File 'lib/origen/application.rb', line 300

def remotes_directory
  workspace_manager.remotes_directory
end

#require_environment!Object



269
270
271
# File 'lib/origen/application.rb', line 269

def require_environment!
  Origen.deprecate 'Calling app.require_environment! is no longer required, the app environment is now automtically loaded when Origen.app is called'
end

#revision_controller(options = {}) ⇒ Object Also known as: rc

Returns a revision controller instance (e.g. Origen::RevisionControl::Git) which has been configured to point to the local workspace and the remote repository as defined by Origen.app.config.rc_url. If the revision control URL has not been defined, or it does not resolve to a recognized revision control system, then this method will return nil.



216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
# File 'lib/origen/application.rb', line 216

def revision_controller(options = {})
  if current?
    if config.rc_url
      begin
        if config.rc_url =~ /^sync:/
          @revision_controller ||= RevisionControl::DesignSync.new(
            local:  root,
            remote: config.rc_url
          )
        elsif config.rc_url =~ /git/
          @revision_controller ||= RevisionControl::Git.new(
            local:                  root,
            # If a workspace is based on a fork of the master repo, config.rc_url may not
            # be correct
            remote:                 (options[:uninitialized] ? config.rc_url : (RevisionControl::Git.origin || config.rc_url)),
            allow_local_adjustment: true
          )

        end
      # The rc_url has been defined, but the initial app checkin has not been done yet
      rescue RevisionControlUninitializedError
        @revision_controller = nil
      end
    elsif config.vault
      @revision_controller ||= RevisionControl::DesignSync.new(
        local:  root,
        remote: config.vault
      )
    end
  else
    fail "Only the top-level application has a revision controller! #{name} is a plugin"
  end
end

#rootObject

Returns a full path to the root directory of the given application

If the application instance is a plugin then this will point to where the application is installed within the imports directory



277
278
279
# File 'lib/origen/application.rb', line 277

def root
  @root
end

#root=(val) ⇒ Object



265
266
267
# File 'lib/origen/application.rb', line 265

def root=(val)
  @root = Pathname.new(val)
end

#runnerObject



685
686
687
# File 'lib/origen/application.rb', line 685

def runner
  @runner ||= Runner.new
end

#server_dataObject

Returns the server data packet available for the given application, returns nil if none is found



376
377
378
379
380
381
382
# File 'lib/origen/application.rb', line 376

def server_data
  if name == :origen
    @server_data ||= Origen.client.origen
  else
    @server_data ||= Origen.client.plugins.find { |p| p[:origen_name].downcase == name.to_s.downcase }
  end
end

#session(reload = false) ⇒ Object



713
714
715
716
717
718
719
720
721
722
723
724
725
# File 'lib/origen/application.rb', line 713

def session(reload = false)
  if current?
    @session = nil if reload
    @session ||= Database::KeyValueStores.new(self, persist: false)
  else
    puts "All plugins should use the top-level application's session store, i.e. use:"
    puts "  Origen.app.session.#{name}"
    puts 'instead of:'
    puts '  Origen.app!.session'
    puts
    exit 1
  end
end

#set_dynamic_resource(name, value) ⇒ Object



948
949
950
# File 'lib/origen/application.rb', line 948

def set_dynamic_resource(name, value)
  dynamic_resource(name, value, set: true)
end

#statisticsObject Also known as: stats



736
737
738
# File 'lib/origen/application.rb', line 736

def statistics
  runner.statistics
end

#subscribers_devObject

Returns an array of users who have subscribed for development release notifications for the given application on the website



366
367
368
369
370
371
372
# File 'lib/origen/application.rb', line 366

def subscribers_dev
  if server_data
    @subscribers_dev ||= server_data[:subscribers_dev].map { |u| User.new(u[:core_id]) }
  else
    []
  end
end

#subscribers_prodObject

Returns an array of users who have subscribed for production release notifications for the given application on the website



356
357
358
359
360
361
362
# File 'lib/origen/application.rb', line 356

def subscribers_prod
  if server_data
    @subscribers_prod ||= server_data[:subscribers_prod].map { |u| User.new(u[:core_id]) }
  else
    []
  end
end

#targetObject



673
674
675
# File 'lib/origen/application.rb', line 673

def target
  @target ||= Target.new
end

#target_instantiated?Boolean

Returns:

  • (Boolean)


999
1000
1001
# File 'lib/origen/application.rb', line 999

def target_instantiated?
  @target_instantiated
end

#target_loading?Boolean

Returns:

  • (Boolean)


1003
1004
1005
# File 'lib/origen/application.rb', line 1003

def target_loading?
  @target_loading || false
end

#testerObject



799
800
801
# File 'lib/origen/application.rb', line 799

def tester
  dynamic_resource(:tester, []).first
end

#tester=(obj) ⇒ Object



803
804
805
806
807
808
809
# File 'lib/origen/application.rb', line 803

def tester=(obj)
  # if tester && obj
  #  raise "You can only instantiate 1 tester, you have already created an instance of #{tester.class}}"
  # end
  $tester = obj
  set_dynamic_resource(:tester, [obj])
end

#top_levelObject

Returns the current top-level object (the DUT)



431
432
433
434
435
436
# File 'lib/origen/application.rb', line 431

def top_level
  @top_level ||= begin
    t = toplevel_listeners.first
    t.controller ? t.controller : t if t
  end
end

#toplevel_listenersObject



756
757
758
# File 'lib/origen/application.rb', line 756

def toplevel_listeners
  dynamic_resource(:toplevel_listeners, [])
end

#unload_target!Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Not a clean unload, but allows objects to be re-instantiated for testing



927
928
929
930
931
932
933
# File 'lib/origen/application.rb', line 927

def unload_target!
  listeners_for(:before_load_target).each(&:before_load_target)
  clear_dynamic_resources
  clear_dynamic_resources(:static)
  Origen::Pins.clear_pin_aliases
  $dut = nil
end

#version(options = {}) ⇒ Object



470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
# File 'lib/origen/application.rb', line 470

def version(options = {})
  @version = nil if options[:refresh]
  return @version if @version

  if Origen.running_globally?
    @version = Origen.version
  else
    load File.join(root, 'config', 'version.rb')
    if defined? eval(namespace)::VERSION
      @version = Origen::VersionString.new(eval(namespace)::VERSION)
    else
      # The eval of the class is required here as somehow when plugins are imported under the old
      # imports system and with the old version file format we can end up with two copies of the
      # same class constant. Don't understand it, but it is fixed with the move to gems and the
      # namespace-based version file format.
      @version = Origen::VersionString.new(eval(self.class.to_s)::VERSION)
    end
  end
  @version
end

#version_trackerObject



697
698
699
# File 'lib/origen/application.rb', line 697

def version_tracker
  @version_tracker ||= VersionTracker.new
end

#versionsObject



727
728
729
# File 'lib/origen/application.rb', line 727

def versions
  version_tracker.versions
end

#with_doc_tester(options = {}) ⇒ Object

Any attempts to instantiate a tester within the give block will be forced to instantiate an Origen::Tester::Doc instance



781
782
783
784
785
786
787
788
789
# File 'lib/origen/application.rb', line 781

def with_doc_tester(options = {})
  @with_doc_tester = true
  if options[:html]
    @with_html_doc_tester = true
  end
  yield
  @with_doc_tester = false
  @with_html_doc_tester = false
end

#with_doc_tester?Boolean

Returns:

  • (Boolean)


791
792
793
# File 'lib/origen/application.rb', line 791

def with_doc_tester?
  @with_doc_tester
end

#with_html_doc_tester?Boolean

Returns:

  • (Boolean)


795
796
797
# File 'lib/origen/application.rb', line 795

def with_html_doc_tester?
  @with_html_doc_tester
end

#workspace_managerObject



701
702
703
# File 'lib/origen/application.rb', line 701

def workspace_manager
  @workspace_manager ||= WorkspaceManager.new
end