Class: Origen::Specs::Documentation

Inherits:
Object
  • Object
show all
Defined in:
lib/origen/specs/documentation.rb

Overview

This class is used to store documentation map that the user can change

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(header_info = {}, selection = {}, applicable_devs = [], link = nil) ⇒ Documentation

Initialize the Class



51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/origen/specs/documentation.rb', line 51

def initialize(header_info = {}, selection = {}, applicable_devs = [], link = nil)
  @level = header_info[:level]
  @section = header_info[:section]
  @subsection = header_info[:subsection]
  @interface = selection[:interface]
  @mode = selection[:mode]
  @type = selection[:type]
  @sub_type = selection[:sub_type]
  @audience = selection[:audience]
  @applicable_devices = applicable_devs
  @link = link
end

Instance Attribute Details

#applicable_devices ⇒ Object

Applicable Devices for the map



48
49
50
# File 'lib/origen/specs/documentation.rb', line 48

def applicable_devices
  @applicable_devices
end

#audience ⇒ Object

Audience is part of the 4-D Hash for the Tables. Corresponds to Spec 4-D Hash



42
43
44
# File 'lib/origen/specs/documentation.rb', line 42

def audience
  @audience
end

#interface ⇒ Object

Exhibit References that should be referenced within the table title



24
25
26
# File 'lib/origen/specs/documentation.rb', line 24

def interface
  @interface
end

#level ⇒ Object

Level that Section is at. Allows for a key to be found.



6
7
8
# File 'lib/origen/specs/documentation.rb', line 6

def level
  @level
end

DITA Formatted Text that appears before the table



45
46
47
# File 'lib/origen/specs/documentation.rb', line 45

def link
  @link
end

#mode ⇒ Object

Mode is part of the 4-D Hash for the Tables. Corresponds to Spec 4-D Hash



27
28
29
# File 'lib/origen/specs/documentation.rb', line 27

def mode
  @mode
end

#section ⇒ Object

This is the Section Header for the Documentation Map. Usually these are main headers Examples:

  1. Overall DC Electricals II. General AC Charactertistics III. Power Sequencing


13
14
15
# File 'lib/origen/specs/documentation.rb', line 13

def section
  @section
end

#sub_type ⇒ Object

SubType is part of the 4-D Hash for the Tables. Corresponds to Spec 4-D Hash



39
40
41
# File 'lib/origen/specs/documentation.rb', line 39

def sub_type
  @sub_type
end

#subsection ⇒ Object

This is the subsection header for the Documentation Map. These are found under main headers Examples

  1. Overall DC electrical
  2. Absolute Maximum Ratings
  3. Recommend Operating Conditions
  4. Output Driver


21
22
23
# File 'lib/origen/specs/documentation.rb', line 21

def subsection
  @subsection
end

#type ⇒ Object

Type is part of the 4-D Hash for the Tables. Corresponds to Spec 4-D Hash Usual values

  • DC -> Direct Current
  • AC -> Alternate Current
  • Temp -> Temperature
  • Supply -> Supply


36
37
38
# File 'lib/origen/specs/documentation.rb', line 36

def type
  @type
end